@@ -47,7 +47,6 @@
 #include <qb/qbutil.h>
 #include <qb/qblog.h>
 
-#include <corosync/list.h>
 #include <corosync/logsys.h>
 
 /*
@@ -380,8 +379,6 @@ int _logsys_system_setup(
 	qb_log_ctl(QB_LOG_BLACKBOX, QB_LOG_CONF_ENABLED, QB_FALSE);
 
 	if (logsys_format_set(NULL) == -1) {
-		pthread_mutex_unlock (&logsys_config_mutex);
-
 		return -1;
 	}
 
@@ -570,17 +567,25 @@ int logsys_config_file_set (
 static void
 logsys_file_format_get(char* file_format, int buf_len)
 {
-	char *per_t;
+	char *format_buffer_start;
+	char *str_pos;
+
 	file_format[0] = '\0';
-	per_t = strstr(format_buffer, "%t");
-	if (per_t) {
-		strcpy(file_format, "%t [%P] %H %N");
-		per_t += 2;
-		strncat(file_format, per_t, buf_len - strlen("%t [%P] %H %N"));
-	} else {
-		strcpy(file_format, "[%P] %H %N");
-		strncat(file_format, format_buffer, buf_len - strlen("[%P] %H %N"));
+
+	format_buffer_start = format_buffer;
+
+	if ((str_pos = strstr(format_buffer, "%t"))) {
+		strcpy(file_format, "%t ");
+		format_buffer_start = str_pos + 2;
 	}
+
+	if ((str_pos = strstr(format_buffer, "%T"))) {
+		strcpy(file_format, "%T ");
+		format_buffer_start = str_pos + 2;
+	}
+
+	strcat(file_format, "[%P] %H %N");
+	strncat(file_format, format_buffer_start, buf_len - strlen(file_format));
 }
 
 int logsys_format_set (const char *format)
@@ -612,7 +617,7 @@ int logsys_format_set (const char *format)
 	}
 
 	/*
-	 * This just goes through and remove %t and %p from
+	 * This just goes through and remove %t, %T and %p from
 	 * the format string for syslog.
 	 */
 	w = 0;
@@ -625,7 +630,8 @@ int logsys_format_set (const char *format)
 					continue;
 				}
 				if (format_buffer[c] == 't' ||
-				    format_buffer[c] == 'p') {
+				    format_buffer[c] == 'p' ||
+				    format_buffer[c] == 'T') {
 					c++;
 				} else {
 					c = reminder;
@@ -880,3 +886,48 @@ void logsys_blackbox_postfork(void)
 
 	_logsys_config_apply_blackbox();
 }
+
+cs_error_t logsys_reopen_log_files(void)
+{
+	cs_error_t res;
+
+#ifdef HAVE_QB_LOG_FILE_REOPEN
+	int i, j;
+	int num_using_current;
+	int32_t rc;
+
+	res = CS_OK;
+
+	pthread_mutex_lock (&logsys_config_mutex);
+
+	for (i = 0; i <= LOGSYS_MAX_SUBSYS_COUNT; i++) {
+		if (logsys_loggers[i].target_id <= 0 || logsys_loggers[i].logfile == NULL) {
+			continue ;
+		}
+
+		num_using_current = 0;
+		for (j = 0; j <= i; j++) {
+			if (logsys_loggers[i].target_id == logsys_loggers[j].target_id) {
+				num_using_current++;
+			}
+		}
+		if (num_using_current == 1) {
+			/*
+			 * First instance of target file. Reopen it.
+			 */
+			rc = qb_log_file_reopen(logsys_loggers[i].target_id, NULL);
+			if (rc != 0) {
+				LOGSYS_PERROR (-rc, LOGSYS_LEVEL_WARNING,
+				    "Unable to reopen log file %s", logsys_loggers[i].logfile);
+				res = qb_to_cs_error(rc);
+			}
+		}
+	}
+
+	pthread_mutex_unlock (&logsys_config_mutex);
+#else
+	res = CS_ERR_NOT_SUPPORTED;
+#endif
+
+	return (res);
+}
