@@ -81,6 +81,8 @@
 #define MAX_MESSAGES				17
 #define MISS_COUNT_CONST			5
 #define BLOCK_UNLISTED_IPS			1
+/* This constant is not used for knet */
+#define UDP_NETMTU                              1500
 
 /* Currently all but PONG_COUNT match the defaults in libknet.h */
 #define KNET_PING_INTERVAL                      1000
@@ -649,11 +651,11 @@ static int nodelist_byname(icmap_map_t map, const char *find_name, int strip_dom
 			char *dot;
 			dot = strchr(name, '.');
 			if (dot) {
-				namelen = name - dot - 1;
+				namelen = dot - name;
 			}
 		}
 		if (strncmp(find_name, name, namelen) == 0 &&
-		    strlen(find_name) == strlen(name)) {
+		    strlen(find_name) == namelen) {
 			icmap_iter_finalize(iter);
 			return node_pos;
 		}
@@ -2099,7 +2101,7 @@ int totem_config_validate (
 			totem_config->net_mtu = KNET_MAX_PACKET_SIZE;
 		}
 		else {
-			totem_config->net_mtu = 1500;
+			totem_config->net_mtu = UDP_NETMTU;
 		}
 	}
 
