@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2005 MontaVista Software, Inc.
- * Copyright (c) 2006-2012 Red Hat, Inc.
+ * Copyright (c) 2006-2018 Red Hat, Inc.
  *
  * All rights reserved.
  *
@@ -58,29 +58,23 @@
 #include <sys/uio.h>
 #include <limits.h>
 
+#include <qb/qblist.h>
 #include <qb/qbdefs.h>
 #include <qb/qbloop.h>
 
 #include <corosync/sq.h>
-#include <corosync/list.h>
 #include <corosync/swab.h>
 #define LOGSYS_UTILS_ONLY 1
 #include <corosync/logsys.h>
 #include "totemudpu.h"
 
 #include "util.h"
-#include "totemcrypto.h"
-
-#include <nss.h>
-#include <pk11pub.h>
-#include <pkcs11.h>
-#include <prerror.h>
 
 #ifndef MSG_NOSIGNAL
 #define MSG_NOSIGNAL 0
 #endif
 
-#define MCAST_SOCKET_BUFFER_SIZE (TRANSMITS_ALLOWED * FRAME_SIZE_MAX)
+#define MCAST_SOCKET_BUFFER_SIZE (TRANSMITS_ALLOWED * UDP_RECEIVE_FRAME_SIZE_MAX)
 #define NETIF_STATE_REPORT_UP		1
 #define NETIF_STATE_REPORT_DOWN		2
 
@@ -89,15 +83,13 @@
 #define BIND_STATE_LOOPBACK	2
 
 struct totemudpu_member {
-	struct list_head list;
+	struct qb_list_head list;
 	struct totem_ip_address member;
 	int fd;
 	int active;
 };
 
 struct totemudpu_instance {
-	struct crypto_instance *crypto_inst;
-
 	qb_loop_t *totemudpu_poll_handle;
 
 	struct totem_interface *totem_interface;
@@ -111,11 +103,13 @@ struct totemudpu_instance {
 	void (*totemudpu_deliver_fn) (
 		void *context,
 		const void *msg,
-		unsigned int msg_len);
+		unsigned int msg_len,
+		const struct sockaddr_storage *system_from);
 
 	void (*totemudpu_iface_change_fn) (
 		void *context,
-		const struct totem_ip_address *iface_address);
+		const struct totem_ip_address *iface_address,
+		unsigned int ring_no);
 
 	void (*totemudpu_target_set_completed) (void *context);
 
@@ -145,11 +139,11 @@ struct totemudpu_instance {
 
 	void *udpu_context;
 
-	char iov_buffer[FRAME_SIZE_MAX];
+	char iov_buffer[UDP_RECEIVE_FRAME_SIZE_MAX];
 
 	struct iovec totemudpu_iov_recv;
 
-	struct list_head member_list;
+	struct qb_list_head member_list;
 
 	int stats_sent;
 
@@ -220,14 +214,14 @@ static void totemudpu_instance_initialize (struct totemudpu_instance *instance)
 
 	instance->totemudpu_iov_recv.iov_base = instance->iov_buffer;
 
-	instance->totemudpu_iov_recv.iov_len = FRAME_SIZE_MAX; //sizeof (instance->iov_buffer);
+	instance->totemudpu_iov_recv.iov_len = UDP_RECEIVE_FRAME_SIZE_MAX; //sizeof (instance->iov_buffer);
 
 	/*
 	 * There is always atleast 1 processor
 	 */
 	instance->my_memb_entries = 1;
 
-	list_init (&instance->member_list);
+	qb_list_init (&instance->member_list);
 }
 
 #define log_printf(level, format, args...)		\
@@ -265,34 +259,13 @@ static inline void ucast_sendmsg (
 {
 	struct msghdr msg_ucast;
 	int res = 0;
-	size_t buf_out_len;
-	unsigned char buf_out[FRAME_SIZE_MAX];
 	struct sockaddr_storage sockaddr;
 	struct iovec iovec;
 	int addrlen;
 	int send_sock;
 
-	if (msg_len + crypto_get_current_sec_header_size(instance->crypto_inst) > sizeof(buf_out)) {
-		log_printf(LOGSYS_LEVEL_CRIT, "UDPU message for ucast is too big. Ignoring message");
-
-		return ;
-	}
-
-	/*
-	 * Encrypt and digest the message
-	 */
-	if (crypto_encrypt_and_sign (
-		instance->crypto_inst,
-		(const unsigned char *)msg,
-		msg_len,
-		buf_out,
-		&buf_out_len) != 0) {
-		log_printf(LOGSYS_LEVEL_CRIT, "Error encrypting/signing packet (non-critical)");
-		return;
-	}
-
-	iovec.iov_base = (void *)buf_out;
-	iovec.iov_len = buf_out_len;
+	iovec.iov_base = (void *)msg;
+	iovec.iov_len = msg_len;
 
 	/*
 	 * Build unicast message
@@ -348,49 +321,24 @@ static inline void mcast_sendmsg (
 {
 	struct msghdr msg_mcast;
 	int res = 0;
-	size_t buf_out_len;
-	unsigned char buf_out[FRAME_SIZE_MAX];
 	struct iovec iovec;
 	struct sockaddr_storage sockaddr;
 	int addrlen;
-        struct list_head *list;
+	struct qb_list_head *list;
 	struct totemudpu_member *member;
 
-	if (msg_len + crypto_get_current_sec_header_size(instance->crypto_inst) > sizeof(buf_out)) {
-		log_printf(LOGSYS_LEVEL_CRIT, "UDPU message for mcast is too big. Ignoring message");
-
-		return ;
-	}
-
-	/*
-	 * Encrypt and digest the message
-	 */
-	if (crypto_encrypt_and_sign (
-		instance->crypto_inst,
-		(const unsigned char *)msg,
-		msg_len,
-		buf_out,
-		&buf_out_len) != 0) {
-		log_printf(LOGSYS_LEVEL_CRIT, "Error encrypting/signing packet (non-critical)");
-		return;
-	}
-
-	iovec.iov_base = (void *)buf_out;
-	iovec.iov_len = buf_out_len;
+	iovec.iov_base = (void *)msg;
+	iovec.iov_len = msg_len;
 
 	memset(&msg_mcast, 0, sizeof(msg_mcast));
 	/*
 	 * Build multicast message
 	 */
-
 	if (instance->netif_bind_state == BIND_STATE_REGULAR) {
-		for (list = instance->member_list.next;
-			list != &instance->member_list;
-			list = list->next) {
-				member = list_entry (list,
-						struct totemudpu_member,
-						list);
-
+		qb_list_for_each(list, &(instance->member_list)) {
+			member = qb_list_entry (list,
+				struct totemudpu_member,
+				list);
 				/*
 				 * Do not send multicast message if message is not "flush", member
 				 * is inactive and timeout for sending merge message didn't expired.
@@ -443,7 +391,6 @@ static inline void mcast_sendmsg (
 		 * Transmit multicast message to local unix mcast loop
 		 * An error here is recovered by totemsrp
 		 */
-
 		msg_mcast.msg_name = NULL;
 		msg_mcast.msg_namelen = 0;
 		msg_mcast.msg_iov = (void *)&iovec;
@@ -501,18 +448,15 @@ static struct totemudpu_member *find_member_by_sockaddr(
 	const void *udpu_context,
 	const struct sockaddr *sa)
 {
-	struct list_head *list;
+	struct qb_list_head *list;
 	struct totemudpu_member *member;
 	struct totemudpu_member *res_member;
 	const struct totemudpu_instance *instance = (const struct totemudpu_instance *)udpu_context;
 
 	res_member = NULL;
 
-	for (list = instance->member_list.next;
-		list != &instance->member_list;
-		list = list->next) {
-
-		member = list_entry (list,
+	qb_list_for_each(list, &(instance->member_list)) {
+		member = qb_list_entry (list,
 			struct totemudpu_member,
 			list);
 
@@ -536,7 +480,6 @@ static int net_deliver_fn (
 	struct iovec *iovec;
 	struct sockaddr_storage system_from;
 	int bytes_received;
-	int res = 0;
 	int truncated_packet;
 
 	iovec = &instance->totemudpu_iov_recv;
@@ -580,24 +523,23 @@ static int net_deliver_fn (
 #else
 	/*
 	 * We don't have MSGHDR_FLAGS, but we can (hopefully) safely make assumption that
-	 * if bytes_received == FRAME_SIZE_MAX then packet is truncated
+	 * if bytes_received == UDP_RECEIVE_FRAME_SIZE_MAX then packet is truncated
 	 */
-	if (bytes_received == FRAME_SIZE_MAX) {
+	if (bytes_received == UDP_RECEIVE_FRAME_SIZE_MAX) {
 		truncated_packet = 1;
 	}
 #endif
 
 	if (truncated_packet) {
-		log_printf(instance->totemudpu_log_level_error,
+		log_printf (instance->totemudpu_log_level_error,
 				"Received too big message. This may be because something bad is happening"
 				"on the network (attack?), or you tried join more nodes than corosync is"
 				"compiled with (%u) or bug in the code (bad estimation of "
-				"the FRAME_SIZE_MAX). Dropping packet.", PROCESSOR_COUNT_MAX);
+				"the UDP_RECEIVE_FRAME_SIZE_MAX). Dropping packet.", PROCESSOR_COUNT_MAX);
 		return (0);
 	}
 
 	if (instance->totem_config->block_unlisted_ips &&
-	    instance->netif_bind_state == BIND_STATE_REGULAR &&
 	    find_member_by_sockaddr(instance, (const struct sockaddr *)&system_from) == NULL) {
 		log_printf(instance->totemudpu_log_level_debug, "Packet rejected from %s",
 		    totemip_sa_print((const struct sockaddr *)&system_from));
@@ -605,18 +547,6 @@ static int net_deliver_fn (
 		return (0);
 	}
 
-	/*
-	 * Authenticate and if authenticated, decrypt datagram
-	 */
-
-	res = crypto_authenticate_and_decrypt (instance->crypto_inst, iovec->iov_base, &bytes_received);
-	if (res == -1) {
-		log_printf (instance->totemudpu_log_level_security, "Received message has invalid digest... ignoring.");
-		log_printf (instance->totemudpu_log_level_security,
-			"Invalid packet data");
-		iovec->iov_len = FRAME_SIZE_MAX;
-		return 0;
-	}
 	iovec->iov_len = bytes_received;
 
 	/*
@@ -625,9 +555,10 @@ static int net_deliver_fn (
 	instance->totemudpu_deliver_fn (
 		instance->context,
 		iovec->iov_base,
-		iovec->iov_len);
+		iovec->iov_len,
+		&system_from);
 
-	iovec->iov_len = FRAME_SIZE_MAX;
+	iovec->iov_len = UDP_RECEIVE_FRAME_SIZE_MAX;
 	return (0);
 }
 
@@ -751,7 +682,7 @@ static void timer_function_netif_check_timeout (
 				"The network interface [%s] is now up.",
 				totemip_print (&instance->totem_interface->boundto));
 			instance->netif_state_report = NETIF_STATE_REPORT_DOWN;
-			instance->totemudpu_iface_change_fn (instance->context, &instance->my_id);
+			instance->totemudpu_iface_change_fn (instance->context, &instance->my_id, 0);
 		}
 		/*
 		 * Add a timer to check for interface going down in single membership
@@ -769,7 +700,7 @@ static void timer_function_netif_check_timeout (
 		if (instance->netif_state_report & NETIF_STATE_REPORT_DOWN) {
 			log_printf (instance->totemudpu_log_level_notice,
 				"The network interface is down.");
-			instance->totemudpu_iface_change_fn (instance->context, &instance->my_id);
+			instance->totemudpu_iface_change_fn (instance->context, &instance->my_id, 0);
 		}
 		instance->netif_state_report = NETIF_STATE_REPORT_UP;
 
@@ -825,9 +756,7 @@ static int totemudpu_build_sockets_ip (
 	 * Bind to unicast socket used for token send/receives
 	 * This has the side effect of binding to the correct interface
 	 */
-	totemip_totemip_to_sockaddr_convert_with_scopeid(bound_to, instance->totem_interface->ip_port,
-	    &sockaddr, &addrlen, 1);
-
+	totemip_totemip_to_sockaddr_convert(bound_to, instance->totem_interface->ip_port, &sockaddr, &addrlen);
 	while (1) {
 		res = bind (instance->token_socket, (struct sockaddr *)&sockaddr, addrlen);
 		if (res == 0) {
@@ -864,6 +793,50 @@ static int totemudpu_build_sockets_ip (
 	return 0;
 }
 
+int totemudpu_nodestatus_get (void *udpu_context, unsigned int nodeid,
+			      struct totem_node_status *node_status)
+{
+	struct totemudpu_instance *instance = (struct totemudpu_instance *)udpu_context;
+	struct qb_list_head *list;
+	struct totemudpu_member *member;
+
+	qb_list_for_each(list, &(instance->member_list)) {
+		member = qb_list_entry (list,
+			struct totemudpu_member,
+			list);
+
+		if (member->member.nodeid == nodeid) {
+			node_status->nodeid = nodeid;
+			/* reachable is filled in by totemsrp */
+			if (instance->netif_bind_state == BIND_STATE_REGULAR) {
+				node_status->link_status[0].enabled = 1;
+			} else {
+				node_status->link_status[0].enabled = 0;
+			}
+			node_status->link_status[0].connected = node_status->reachable;
+			node_status->link_status[0].mtu = instance->totem_config->net_mtu;
+			strncpy(node_status->link_status[0].src_ipaddr, totemip_print(&member->member), KNET_MAX_HOST_LEN-1);
+		}
+	}
+	return (0);
+}
+
+int totemudpu_ifaces_get (
+	void *net_context,
+	char ***status,
+	unsigned int *iface_count)
+{
+	static char *statuses[INTERFACE_MAX] = {(char*)"OK"};
+
+	if (status) {
+		*status = statuses;
+	}
+	*iface_count = 1;
+
+	return (0);
+}
+
+
 static int totemudpu_build_local_sockets(
 	struct totemudpu_instance *instance)
 {
@@ -969,7 +942,7 @@ static int totemudpu_build_sockets (
 }
 
 /*
- * Totem Network interface - also does encryption/decryption
+ * Totem Network interface
  * depends on poll abstraction, POSIX, IPV4
  */
 
@@ -981,17 +954,22 @@ int totemudpu_initialize (
 	void **udpu_context,
 	struct totem_config *totem_config,
 	totemsrp_stats_t *stats,
-	int interface_no,
 	void *context,
 
 	void (*deliver_fn) (
 		void *context,
 		const void *msg,
-		unsigned int msg_len),
+		unsigned int msg_len,
+		const struct sockaddr_storage *system_from),
 
 	void (*iface_change_fn) (
 		void *context,
-		const struct totem_ip_address *iface_address),
+		const struct totem_ip_address *iface_address,
+		unsigned int ring_no),
+
+	void (*mtu_changed) (
+		void *context,
+		int net_mtu),
 
 	void (*target_set_completed) (
 		void *context))
@@ -1019,27 +997,11 @@ int totemudpu_initialize (
 	instance->totemudpu_subsys_id = totem_config->totem_logging_configuration.log_subsys_id;
 	instance->totemudpu_log_printf = totem_config->totem_logging_configuration.log_printf;
 
-	/*
-	* Initialize random number generator for later use to generate salt
-	*/
-	instance->crypto_inst = crypto_init (totem_config->private_key,
-		totem_config->private_key_len,
-		totem_config->crypto_cipher_type,
-		totem_config->crypto_hash_type,
-		instance->totemudpu_log_printf,
-		instance->totemudpu_log_level_security,
-		instance->totemudpu_log_level_notice,
-		instance->totemudpu_log_level_error,
-		instance->totemudpu_subsys_id);
-	if (instance->crypto_inst == NULL) {
-		free(instance);
-		return (-1);
-	}
 	/*
 	 * Initialize local variables for totemudpu
 	 */
-	instance->totem_interface = &totem_config->interfaces[interface_no];
-	memset (instance->iov_buffer, 0, FRAME_SIZE_MAX);
+	instance->totem_interface = &totem_config->interfaces[0];
+	memset (instance->iov_buffer, 0, UDP_RECEIVE_FRAME_SIZE_MAX);
 
 	instance->totemudpu_poll_handle = poll_handle;
 
@@ -1179,47 +1141,33 @@ extern int totemudpu_iface_check (void *udpu_context)
 
 extern void totemudpu_net_mtu_adjust (void *udpu_context, struct totem_config *totem_config)
 {
-
-	assert(totem_config->interface_count > 0);
-
-	totem_config->net_mtu -= crypto_sec_header_size(totem_config->crypto_cipher_type,
-							totem_config->crypto_hash_type) +
-				 totemip_udpip_header_size(totem_config->interfaces[0].bindnet.family);
+	totem_config->net_mtu -= totemip_udpip_header_size(totem_config->interfaces[0].bindnet.family);
 }
 
-const char *totemudpu_iface_print (void *udpu_context)  {
-	struct totemudpu_instance *instance = (struct totemudpu_instance *)udpu_context;
-	const char *ret_char;
-
-	ret_char = totemip_print (&instance->my_id);
-
-	return (ret_char);
-}
-
-int totemudpu_iface_get (
-	void *udpu_context,
-	struct totem_ip_address *addr)
-{
-	struct totemudpu_instance *instance = (struct totemudpu_instance *)udpu_context;
-	int res = 0;
-
-	memcpy (addr, &instance->my_id, sizeof (struct totem_ip_address));
-
-	return (res);
-}
 
 int totemudpu_token_target_set (
 	void *udpu_context,
-	const struct totem_ip_address *token_target)
+	unsigned int nodeid)
 {
+
 	struct totemudpu_instance *instance = (struct totemudpu_instance *)udpu_context;
+	struct qb_list_head *list;
+	struct totemudpu_member *member;
 	int res = 0;
 
-	memcpy (&instance->token_target, token_target,
-		sizeof (struct totem_ip_address));
+	qb_list_for_each(list, &(instance->member_list)) {
+		member = qb_list_entry (list,
+			struct totemudpu_member,
+			list);
 
-	instance->totemudpu_target_set_completed (instance->context);
+		if (member->member.nodeid == nodeid) {
+			memcpy (&instance->token_target, &member->member,
+				sizeof (struct totem_ip_address));
 
+			instance->totemudpu_target_set_completed (instance->context);
+			break;
+		}
+	}
 	return (res);
 }
 
@@ -1334,7 +1282,7 @@ static int totemudpu_create_sending_socket(
 	/*
 	 * Bind to sending interface
 	 */
-	totemip_totemip_to_sockaddr_convert_with_scopeid(&instance->my_id, 0, &sockaddr, &addrlen, 1);
+	totemip_totemip_to_sockaddr_convert(&instance->my_id, 0, &sockaddr, &addrlen);
 	res = bind (fd, (struct sockaddr *)&sockaddr, addrlen);
 	if (res == -1) {
 		LOGSYS_PERROR (errno, instance->totemudpu_log_level_warning,
@@ -1349,9 +1297,20 @@ error_close_fd:
 	return (-1);
 }
 
+int totemudpu_iface_set (void *net_context,
+	const struct totem_ip_address *local_addr,
+	unsigned short ip_port,
+	unsigned int iface_no)
+{
+	/* Not supported */
+	return (-1);
+}
+
 int totemudpu_member_add (
 	void *udpu_context,
-	const struct totem_ip_address *member)
+	const struct totem_ip_address *local,
+	const struct totem_ip_address *member,
+	int ring_no)
 {
 	struct totemudpu_instance *instance = (struct totemudpu_instance *)udpu_context;
 
@@ -1366,21 +1325,22 @@ int totemudpu_member_add (
 
 	log_printf (LOGSYS_LEVEL_NOTICE, "adding new UDPU member {%s}",
 		totemip_print(member));
-	list_init (&new_member->list);
-	list_add_tail (&new_member->list, &instance->member_list);
+	qb_list_init (&new_member->list);
+	qb_list_add_tail (&new_member->list, &instance->member_list);
 	memcpy (&new_member->member, member, sizeof (struct totem_ip_address));
 	new_member->fd = totemudpu_create_sending_socket(udpu_context, member);
-	new_member->active = 0;
+	new_member->active = 1;
 
 	return (0);
 }
 
 int totemudpu_member_remove (
 	void *udpu_context,
-	const struct totem_ip_address *token_target)
+	const struct totem_ip_address *token_target,
+	int ring_no)
 {
 	int found = 0;
-	struct list_head *list;
+	struct qb_list_head *list;
 	struct totemudpu_member *member;
 
 	struct totemudpu_instance *instance = (struct totemudpu_instance *)udpu_context;
@@ -1388,11 +1348,8 @@ int totemudpu_member_remove (
 	/*
 	 * Find the member to remove and close its socket
 	 */
-	for (list = instance->member_list.next;
-		list != &instance->member_list;
-		list = list->next) {
-
-		member = list_entry (list,
+	qb_list_for_each(list, &(instance->member_list)) {
+		member = qb_list_entry (list,
 			struct totemudpu_member,
 			list);
 
@@ -1418,7 +1375,7 @@ int totemudpu_member_remove (
 	 * Delete the member from the list
 	 */
 	if (found) {
-		list_del (list);
+		qb_list_del (list);
 	}
 
 	instance = NULL;
@@ -1428,16 +1385,13 @@ int totemudpu_member_remove (
 int totemudpu_member_list_rebind_ip (
 	void *udpu_context)
 {
-	struct list_head *list;
+	struct qb_list_head *list;
 	struct totemudpu_member *member;
 
 	struct totemudpu_instance *instance = (struct totemudpu_instance *)udpu_context;
 
-	for (list = instance->member_list.next;
-		list != &instance->member_list;
-		list = list->next) {
-
-		member = list_entry (list,
+	qb_list_for_each(list, &(instance->member_list)) {
+		member = qb_list_entry (list,
 			struct totemudpu_member,
 			list);
 
@@ -1451,45 +1405,6 @@ int totemudpu_member_list_rebind_ip (
 	return (0);
 }
 
-int totemudpu_member_set_active (
-	void *udpu_context,
-	const struct totem_ip_address *member_ip,
-	int active)
-{
-	struct list_head *list;
-	struct totemudpu_member *member;
-	int addr_found = 0;
-
-	struct totemudpu_instance *instance = (struct totemudpu_instance *)udpu_context;
-
-	/*
-	 * Find the member to set active flag
-	 */
-	for (list = instance->member_list.next; list != &instance->member_list;	list = list->next) {
-		member = list_entry (list, struct totemudpu_member, list);
-
-		if (totemip_compare (member_ip, &member->member) == 0) {
-			log_printf(LOGSYS_LEVEL_DEBUG,
-			    "Marking UDPU member %s %s",
-			    totemip_print(&member->member),
-			    (active ? "active" : "inactive"));
-
-			member->active = active;
-			addr_found = 1;
-
-			break;
-		}
-	}
-
-	if (!addr_found) {
-		log_printf(LOGSYS_LEVEL_DEBUG,
-		    "Can't find UDPU member %s (should be marked as %s)",
-			    totemip_print(member_ip),
-			    (active ? "active" : "inactive"));
-	}
-
-	return (0);
-}
 
 static void timer_function_merge_detect_timeout (
 	void *data)
@@ -1527,3 +1442,11 @@ static void totemudpu_stop_merge_detect_timeout(
 	qb_loop_timer_del(instance->totemudpu_poll_handle,
 	    instance->timer_merge_detect_timeout);
 }
+
+int totemudpu_reconfigure (
+	void *udpu_context,
+	struct totem_config *totem_config)
+{
+	/* Not supported */
+	return (-1);
+}
