@@ -43,6 +43,8 @@
 #include <pthread.h>
 #include <limits.h>
 
+#include <corosync/corotypes.h>
+
 #include <qb/qbconfig.h>
 #include <qb/qblog.h>
 
@@ -256,6 +258,14 @@ extern int _logsys_subsys_create (const char *subsys, const char *filename);
  */
 extern int logsys_thread_start (void);
 
+extern void logsys_blackbox_set(int enable);
+
+extern void logsys_blackbox_prefork(void);
+
+extern void logsys_blackbox_postfork(void);
+
+extern cs_error_t logsys_reopen_log_files(void);
+
 /**
  * @brief logsys_subsys_id
  */
@@ -269,6 +279,7 @@ static int logsys_subsys_id __attribute__((unused)) = LOGSYS_MAX_SUBSYS_COUNT;
  * @param syslog_priority
  */
 #define LOGSYS_DECLARE_SYSTEM(name,mode,syslog_facility,syslog_priority)\
+QB_LOG_INIT_DATA(logsys_qb_init);					\
 __attribute__ ((constructor))						\
 static void logsys_system_init (void)					\
 {									\
@@ -279,12 +290,6 @@ static void logsys_system_init (void)					\
 	}								\
 }
 
-#ifdef QB_HAVE_ATTRIBUTE_SECTION
-#define LOGSYS_DECLARE_SECTION assert(__start___verbose != __stop___verbose)
-#else
-#define LOGSYS_DECLARE_SECTION
-#endif
-
 /**
  * @brief The LOGSYS_DECLARE_SUBSYS macro
  * @param subsys
@@ -293,7 +298,6 @@ static void logsys_system_init (void)					\
 __attribute__ ((constructor))						\
 static void logsys_subsys_init (void)					\
 {									\
-	LOGSYS_DECLARE_SECTION;						\
 	logsys_subsys_id =						\
 		_logsys_subsys_create ((subsys), __FILE__);		\
 	if (logsys_subsys_id == -1) {					\
