@@ -51,6 +51,7 @@
 
 #define FRAME_SIZE_MAX		KNET_MAX_PACKET_SIZE
 
+#define CONFIG_STRING_LEN_MAX   128
 /*
  * Estimation of required buffer size for totemudp and totemudpu - it should be at least
  *   sizeof(memb_join) + PROCESSOR_MAX * 2 * sizeof(srp_addr))
@@ -82,6 +83,7 @@ struct totem_interface {
 	struct totem_ip_address bindnet;
 	struct totem_ip_address boundto;
 	struct totem_ip_address mcast_addr;
+	struct totem_ip_address local_ip;
 	uint16_t ip_port;
 	uint16_t ttl;
 	uint8_t  configured;
@@ -149,6 +151,11 @@ struct memb_ring_id {
 	unsigned long long seq;
 } __attribute__((packed));
 
+typedef enum {
+	CRYPTO_RECONFIG_PHASE_ACTIVATE = 1,
+	CRYPTO_RECONFIG_PHASE_CLEANUP = 2,
+} cfg_message_crypto_reconfig_phase_t;
+
 struct totem_config {
 	int version;
 
@@ -211,17 +218,19 @@ struct totem_config {
 
 	unsigned int max_messages;
 
-	const char *vsf_type;
-
 	unsigned int broadcast_use;
 
-	char *crypto_model;
+	char crypto_model[CONFIG_STRING_LEN_MAX];
 
-	char *crypto_cipher_type;
+	char crypto_cipher_type[CONFIG_STRING_LEN_MAX];
 
-	char *crypto_hash_type;
+	char crypto_hash_type[CONFIG_STRING_LEN_MAX];
 
-	char *knet_compression_model;
+	int crypto_index; /* Num of crypto config currently loaded into knet ( 1 or 2 ) */
+
+	int crypto_changed; /* Has crypto changed since last time? (it's expensive to reload) */
+
+	char knet_compression_model[CONFIG_STRING_LEN_MAX];
 
 	uint32_t knet_compression_threshold;
 
@@ -244,6 +253,25 @@ struct totem_config {
 	    unsigned int nodeid);
 };
 
+/*
+ * Node status returned from the API
+ * Usually the same as the cfg version (except for
+ * link_status)
+ */
+#define TOTEM_NODE_STATUS_STRUCTURE_VERSION 1
+struct totem_node_status {
+	uint32_t version; /* Structure version */
+	unsigned int nodeid;
+	uint8_t reachable;
+	uint8_t remote;
+	uint8_t external;
+	uint8_t onwire_min;
+	uint8_t onwire_max;
+	uint8_t onwire_ver;
+	struct knet_link_status link_status[KNET_MAX_LINK];
+};
+
+
 #define TOTEM_CONFIGURATION_TYPE
 enum totem_configuration_type {
 	TOTEM_CONFIGURATION_REGULAR,
