@@ -25,6 +25,10 @@
 /* Easier than including the config file with a ton of conflicting dependencies */
 extern int coroparse_configparse (icmap_map_t config_map, const char **error_string);
 extern int corosync_log_config_read (const char **error_string);
+static int stdin_read_fn(int32_t fd, int32_t revents, void *data);
+
+/* 'Keep the compiler happy' time */
+const char *corosync_get_config_file(void);
 
 /* One of these per partition */
 struct vq_partition {
@@ -53,23 +57,32 @@ static qb_loop_t *poll_loop;
 static int autofence;
 static int check_for_quorum;
 static FILE *output_file;
-static int nosync;
+static int sync_cmds = 1;
 static qb_loop_timer_handle kb_timer;
-static ssize_t wait_count;
-static ssize_t wait_count_to_unblock;
+static int waiting_for_sync = 0;
+static int is_tty;
+static int assert_on_timeout;
+static uint64_t command_timeout = 250000000L;
 
 static struct vq_node *find_by_pid(pid_t pid);
 static void send_partition_to_nodes(struct vq_partition *partition, int newring);
-static void start_kb_input(void);
 static void start_kb_input_timeout(void *data);
+static void finish_wait_timeout(void *data);
 
 #ifndef HAVE_READLINE_READLINE_H
 #define INPUT_BUF_SIZE 1024
 static char input_buf[INPUT_BUF_SIZE];
 static size_t input_buf_term = 0;
-static int is_tty;
 #endif
 
+/* 'Keep the compiler happy' time */
+static char corosync_config_file[PATH_MAX + 1] = COROSYSCONFDIR "/corosync.conf";
+
+const char *corosync_get_config_file(void)
+{
+	return (corosync_config_file);
+}
+
 /* Tell all non-quorate nodes to quit */
 static void force_fence(void)
 {
@@ -104,19 +117,19 @@ static void print_quorum_state(struct vq_node *node)
 	int i;
 
 	if (node->last_quorate < 0) {
-		fprintf(output_file, "%d:%02d: q=UNINITIALIZED\n",
+		fprintf(output_file, "%d:" CS_PRI_NODE_ID ": q=UNINITIALIZED\n",
 			node->partition->num, node->nodeid);
 		return;
 	}
 
-	fprintf(output_file, "%d:%02d: q=%d ring=[%d/%lld] ", node->partition->num, node->nodeid, node->last_quorate,
-		node->last_ring_id.rep.nodeid, node->last_ring_id.seq);
+	fprintf(output_file, "%d:" CS_PRI_NODE_ID ": q=%d ring=[" CS_PRI_RING_ID "] ", node->partition->num, node->nodeid, node->last_quorate,
+		node->last_ring_id.nodeid, (uint64_t)node->last_ring_id.seq);
 	fprintf(output_file, "nodes=[");
 	for (i = 0; i < node->last_view_list_entries; i++) {
 		if (i) {
 			fprintf(output_file, " ");
 		}
-		fprintf(output_file, "%d", node->last_view_list[i]);
+		fprintf(output_file, CS_PRI_NODE_ID, node->last_view_list[i]);
 	}
 	fprintf(output_file, "]\n");
 
@@ -125,13 +138,78 @@ static void print_quorum_state(struct vq_node *node)
 static void propogate_vq_message(struct vq_node *vqn, const char *msg, int len)
 {
 	struct vq_node *other_vqn;
+	ssize_t write_res;
 
 	/* Send it to everyone in that node's partition (including itself) */
 	TAILQ_FOREACH(other_vqn, &vqn->partition->nodelist, entries) {
-		write(other_vqn->fd, msg, len);
+		write_res = write(other_vqn->fd, msg, len);
+		/*
+		 * Read counterpart is not ready for receiving non-complete message so
+		 * ensure all required information was send.
+		 */
+		assert(write_res == len);
 	}
 }
 
+
+static void cmd_show_prompt_if_needed(void)
+{
+	qb_loop_timer_del(poll_loop, kb_timer);
+	if (is_tty) {
+		printf("vqsim> ");
+		fflush(stdout);
+	} else {
+		printf("#vqsim> ");
+		fflush(stdout);
+	}
+
+}
+
+void resume_kb_input(int show_status)
+{
+	/* If running synchronously, we don't display
+	   the quorum messages as they come in. So run 'show' commamnd
+	*/
+	if (show_status && waiting_for_sync) {
+		cmd_show_node_states();
+	}
+
+	waiting_for_sync = 0;
+
+	if (qb_loop_poll_add(poll_loop,
+			     QB_LOOP_MED,
+			     STDIN_FILENO,
+			     POLLIN | POLLERR,
+			     NULL,
+			     stdin_read_fn)) {
+		if (errno != EEXIST) {
+			perror("qb_loop_poll_add1 returned error");
+		}
+	}
+	/* Always shows the prompt here, cos we cleared waiting_for_sync */
+	cmd_show_prompt_if_needed();
+}
+
+/* Return true (1) if all nodes in each partition have the same ring id, false(0) otherwise */
+static int all_nodes_consistent(void)
+{
+	int i;
+	struct vq_node *vqn;
+	struct memb_ring_id last_ring_id;
+
+	for (i=0; i<MAX_PARTITIONS; i++) {
+		memset(&last_ring_id, 0, sizeof(last_ring_id));
+		TAILQ_FOREACH(vqn, &partitions[i].nodelist, entries) {
+			if (last_ring_id.seq &&
+			    last_ring_id.seq != vqn->last_ring_id.seq) {
+				return 0;
+			}
+			last_ring_id.seq = vqn->last_ring_id.seq;
+		}
+	}
+	return 1;
+}
+
 static int vq_parent_read_fn(int32_t fd, int32_t revents, void *data)
 {
 	char msgbuf[8192];
@@ -144,19 +222,34 @@ static int vq_parent_read_fn(int32_t fd, int32_t revents, void *data)
 		msglen = read(fd, msgbuf, sizeof(msgbuf));
 		if (msglen < 0) {
 			perror("read failed");
-		}
-
-		if (msglen > 0) {
+		} else if (msglen < sizeof(*msg)) {
+			fprintf(stderr, "Received message is too short\n");
+		} else {
 			msg = (void*)msgbuf;
 			switch (msg->type) {
 			case VQMSG_QUORUM:
-				if (!nosync && --wait_count_to_unblock <= 0)
-					qb_loop_timer_del(poll_loop, kb_timer);
 				qmsg = (void*)msgbuf;
+				/*
+				 * Check length of message.
+				 * SOCK_SEQPACKET is used so this check is not strictly needed.
+				 */
+				if (msglen < sizeof(*qmsg) ||
+				    qmsg->view_list_entries > MAX_NODES ||
+				    msglen < sizeof(*qmsg) + sizeof(qmsg->view_list[0]) * qmsg->view_list_entries) {
+					fprintf(stderr, "Received quorum message is too short or corrupted\n");
+					return (0);
+				}
 				save_quorum_state(vqn, qmsg);
-				print_quorum_state(vqn);
-				if (!nosync && wait_count_to_unblock <= 0)
-					start_kb_input();
+				if (!sync_cmds) {
+					print_quorum_state(vqn);
+				}
+
+				/* Have the partitions stabilised? */
+				if (sync_cmds && waiting_for_sync &&
+				    all_nodes_consistent()) {
+					qb_loop_timer_del(poll_loop, kb_timer);
+					resume_kb_input(sync_cmds);
+				}
 				break;
 			case VQMSG_EXEC:
 				/* Message from votequorum, pass around the partition */
@@ -172,7 +265,7 @@ static int vq_parent_read_fn(int32_t fd, int32_t revents, void *data)
 		}
 	}
 	if (revents == POLLERR) {
-		fprintf(stderr, "pollerr on %d\n", vqn->nodeid);
+		fprintf(stderr, "pollerr on " CS_PRI_NODE_ID "\n", vqn->nodeid);
 	}
 	return 0;
 }
@@ -192,7 +285,7 @@ static int read_corosync_conf(void)
 	logsys_format_set(NULL);
 	res = coroparse_configparse(icmap_get_global_map(), &error_string);
 	if (res == -1) {
-		log_printf (LOGSYS_LEVEL_INFO, "Error loading corosyc.conf %s", error_string);
+		log_printf (LOGSYS_LEVEL_INFO, "Error loading corosync.conf %s", error_string);
 		return -1;
 	}
 	else {
@@ -209,6 +302,7 @@ static int read_corosync_conf(void)
 	        log_printf (LOGSYS_LEVEL_ERROR, "Can't initialize log thread");
 		return -1;
 	}
+
 	return 0;
 }
 
@@ -221,8 +315,6 @@ static void remove_node(struct vq_node *node)
 	TAILQ_REMOVE(&part->nodelist, node, entries);
 	free(node);
 
-	wait_count--;
-
 	/* Rebuild quorum */
 	send_partition_to_nodes(part, 1);
 }
@@ -250,7 +342,7 @@ static int32_t sigchld_handler(int32_t sig, void *data)
 				sprintf(text, "(exit code %d)", WEXITSTATUS(status));
 				break;
 			}
-			printf("%d:%02d Quit %s\n", vqn->partition->num, vqn->nodeid, exit_status);
+			printf("%d:" CS_PRI_NODE_ID ": Quit %s\n", vqn->partition->num, vqn->nodeid, exit_status);
 
 			remove_node(vqn);
 		}
@@ -261,7 +353,7 @@ static int32_t sigchld_handler(int32_t sig, void *data)
 	if (WIFSIGNALED(status)) {
 		vqn = find_by_pid(pid);
 		if (vqn) {
-			printf("%d:%02d exited on signal %d%s\n", vqn->partition->num, vqn->nodeid, WTERMSIG(status), WCOREDUMP(status)?" (core dumped)":"");
+			printf("%d:" CS_PRI_NODE_ID " exited on signal %d%s\n", vqn->partition->num, vqn->nodeid, WTERMSIG(status), WCOREDUMP(status)?" (core dumped)":"");
 			remove_node(vqn);
 		}
 		else {
@@ -287,7 +379,7 @@ static void send_partition_to_nodes(struct vq_partition *partition, int newring)
 	TAILQ_FOREACH(vqn, &partition->nodelist, entries) {
 		nodelist[nodes++] = vqn->nodeid;
 		if (first) {
-			partition->ring_id.rep.nodeid = vqn->nodeid;
+			partition->ring_id.nodeid = vqn->nodeid;
 			first = 0;
 		}
 	}
@@ -303,7 +395,7 @@ static void init_partitions(void)
 
 	for (i=0; i<MAX_PARTITIONS; i++) {
 		TAILQ_INIT(&partitions[i].nodelist);
-		partitions[i].ring_id.rep.nodeid = 1000+i;
+		partitions[i].ring_id.nodeid = 1000+i;
 		partitions[i].ring_id.seq = 0;
 		partitions[i].num = i;
 	}
@@ -315,20 +407,13 @@ static pid_t create_node(int nodeid, int partno)
 
 	newvq = malloc(sizeof(struct vq_node));
 	if (newvq) {
-		if (!nosync) {
-			/* Number of expected "quorum" vq messages is a square
-			   of the total nodes count, so increment the node
-			   counter and set new square of this value as
-			   a "to observe" counter */
-			wait_count++;
-			wait_count_to_unblock = wait_count * wait_count;
-		}
 		newvq->last_quorate = -1;  /* mark "uninitialized" */
 		newvq->instance = vq_create_instance(poll_loop, nodeid);
 		if (!newvq->instance) {
 			fprintf(stderr,
-			        "ERR: could not create vq instance nodeid %d\n",
+			        "ERR: could not create vq instance nodeid " CS_PRI_NODE_ID "\n",
 				nodeid);
+			free(newvq);
 			return (pid_t) -1;
 		}
 		newvq->partition = &partitions[partno];
@@ -382,7 +467,7 @@ static size_t create_nodes_from_config(void)
 			pid = create_node(nodeid, 0);
 			if (pid == (pid_t) -1) {
 				fprintf(stderr,
-					"ERR: nodeid %d could not be spawned\n",
+					"ERR: nodeid " CS_PRI_NODE_ID " could not be spawned\n",
 					nodeid);
 				exit(1);
 			}
@@ -426,29 +511,39 @@ static struct vq_node *find_by_pid(pid_t pid)
 }
 
 /* Routines called from the parser */
-void cmd_start_new_node(int nodeid, int partition)
+
+
+/*
+ * The parser calls this before running a command where
+ * we might have to wait for a result to come back.
+ */
+void cmd_start_sync_command()
+{
+	if (sync_cmds) {
+		qb_loop_poll_del(poll_loop, STDIN_FILENO);
+		qb_loop_timer_add(poll_loop,
+				  QB_LOOP_MED,
+				  command_timeout,
+				  NULL,
+				  finish_wait_timeout,
+				  &kb_timer);
+		waiting_for_sync = 1;
+	}
+}
+
+int cmd_start_new_node(int nodeid, int partition)
 {
 	struct vq_node *node;
 
 	node = find_node(nodeid);
 	if (node) {
-		fprintf(stderr, "ERR: nodeid %d already exists in partition %d\n", nodeid, node->partition->num);
-		return;
+		fprintf(stderr, "ERR: nodeid " CS_PRI_NODE_ID " already exists in partition %d\n", nodeid, node->partition->num);
+		return -1;
 	}
-	qb_loop_poll_del(poll_loop, STDIN_FILENO);
-	create_node(nodeid, partition);
-	if (!nosync) {
-		/* Delay kb input handling by 0.25 second when we've just
-		   added a node; expect that the delay will be cancelled
-		   substantially earlier once it has reported its quorum info
-		   (the delay is in fact a failsafe input enabler here) */
-		qb_loop_timer_add(poll_loop,
-				  QB_LOOP_MED,
-				  250000000,
-				  NULL,
-				  start_kb_input_timeout,
-				  &kb_timer);
+	if (create_node(nodeid, partition) == -1) {
+		return -1;
 	}
+	return 0;
 }
 
 void cmd_stop_all_nodes()
@@ -476,20 +571,21 @@ void cmd_show_node_states()
 	fprintf(output_file, "#autofence: %s\n", autofence?"on":"off");
 }
 
-void cmd_stop_node(int nodeid)
+int cmd_stop_node(int nodeid)
 {
 	struct vq_node *node;
 
 	node = find_node(nodeid);
 	if (!node) {
-		fprintf(stderr, "ERR: nodeid %d is not up\n", nodeid);
-		return;
+		fprintf(stderr, "ERR: nodeid " CS_PRI_NODE_ID " is not up\n", nodeid);
+		return -1;
 	}
 
 	/* Remove processor */
 	vq_quit(node->instance);
 
 	/* Node will be removed when the child process exits */
+	return 0;
 }
 
 /* Move all nodes in 'nodelist' into partition 'partition' */
@@ -497,6 +593,13 @@ void cmd_move_nodes(int partition, int num_nodes, int *nodelist)
 {
 	int i;
 	struct vq_node *node;
+	struct vq_node *vqn;
+	int total_nodes = num_nodes;
+
+	/* Work out the number of nodes affected */
+	TAILQ_FOREACH(vqn, &partitions[partition].nodelist, entries) {
+		total_nodes++;
+	}
 
 	for (i=0; i<num_nodes; i++) {
 		node = find_node(nodelist[i]);
@@ -510,7 +613,7 @@ void cmd_move_nodes(int partition, int num_nodes, int *nodelist)
 			node->partition = &partitions[partition];
 		}
 		else {
-			printf("ERR: node %d does not exist\n", nodelist[i]);
+			printf("ERR: node " CS_PRI_NODE_ID " does not exist\n", nodelist[i]);
 		}
 	}
 }
@@ -520,15 +623,11 @@ void cmd_join_partitions(int part1, int part2)
 {
 	struct vq_node *vqn;
 
-	/* TAILQ_FOREACH is not delete safe *sigh* */
-retry:
-	TAILQ_FOREACH(vqn, &partitions[part2].nodelist, entries) {
-
+	while (!TAILQ_EMPTY(&partitions[part2].nodelist)) {
+		vqn = TAILQ_FIRST(&partitions[part2].nodelist);
 		TAILQ_REMOVE(&vqn->partition->nodelist, vqn, entries);
 		TAILQ_INSERT_TAIL(&partitions[part1].nodelist, vqn, entries);
 		vqn->partition = &partitions[part1];
-
-		goto retry;
 	}
 }
 
@@ -538,6 +637,18 @@ void cmd_set_autofence(int onoff)
 	fprintf(output_file, "#autofence: %s\n", onoff?"on":"off");
 }
 
+void cmd_set_sync(int onoff)
+{
+	autofence = onoff;
+	fprintf(output_file, "#sync: %s\n", onoff?"on":"off");
+	sync_cmds = onoff;
+}
+
+void cmd_set_assert(int onoff)
+{
+	assert_on_timeout = onoff;
+}
+
 void cmd_update_all_partitions(int newring)
 {
 	int i;
@@ -558,6 +669,24 @@ void cmd_qdevice_poll(int nodeid, int onoff)
 	}
 }
 
+/* If we get called then a command has timed-out */
+static void finish_wait_timeout(void *data)
+{
+	if (command_timeout) {
+		fprintf(stderr, "ERR: Partition(s) not stable within timeout\n");
+		if (assert_on_timeout) {
+			exit(2);
+		}
+	}
+
+	resume_kb_input(sync_cmds);
+}
+
+void cmd_set_timeout(uint64_t seconds)
+{
+	command_timeout = seconds * QB_TIME_NS_IN_MSEC;
+}
+
 /* ---------------------------------- */
 
 #ifndef HAVE_READLINE_READLINE_H
@@ -585,11 +714,6 @@ static void dummy_read_char()
 
 	parse_input_command((c == EOF) ? NULL : input_buf);
 	input_buf_term = 0;
-
-	if (is_tty) {
-		printf("vqsim> ");
-		fflush(stdout);
-	}
 }
 #endif
 
@@ -604,70 +728,48 @@ static int stdin_read_fn(int32_t fd, int32_t revents, void *data)
 	return 0;
 }
 
-static void start_kb_input(void)
-{
-	wait_count_to_unblock = 0;
-
-#ifdef HAVE_READLINE_READLINE_H
-	/* Readline will deal with completed lines when they arrive */
-	rl_callback_handler_install("vqsim> ", parse_input_command);
-#else
-	if (is_tty) {
-		printf("vqsim> ");
-		fflush(stdout);
-	}
-#endif
-
-	/* Send stdin to readline */
-	if (qb_loop_poll_add(poll_loop,
-			     QB_LOOP_MED,
-			     STDIN_FILENO,
-			     POLLIN | POLLERR,
-			     NULL,
-			     stdin_read_fn)) {
-		if (errno != EEXIST) {
-			perror("qb_loop_poll_add1 returned error");
-		}
-	}
-}
 
 static void start_kb_input_timeout(void *data)
 {
-//	fprintf(stderr, "Waiting for nodes to report status timed out\n");
-	start_kb_input();
+	resume_kb_input(1);
 }
 
 static void usage(char *program)
 {
 	printf("Usage:\n");
 	printf("\n");
-	printf("%s [-f <config-file>] [-o <output-file>]\n", program);
+	printf("%s [-c <config-file>] [-o <output-file>]\n", program);
 	printf("\n");
-	printf("    -f     config file. defaults to /etc/corosync/corosync.conf\n");
+	printf("    -c     config file. defaults to /etc/corosync/corosync.conf\n");
 	printf("    -o     output file. defaults to stdout\n");
 	printf("    -n     no synchronization (on adding a node)\n");
 	printf("    -h     display this help text\n");
 	printf("\n");
+	printf("%s always takes input from STDIN, but cannot use a file.\n", program);
+	printf("If you want to script it then use\n cat | %s\n", program);
+	printf("\n");
 }
 
 int main(int argc, char **argv)
 {
 	qb_loop_signal_handle sigchld_qb_handle;
 	int ch;
-	char *config_file_name = NULL;
 	char *output_file_name = NULL;
-	char envstring[PATH_MAX];
 
-	while ((ch = getopt (argc, argv, "f:o:nh")) != EOF) {
+	while ((ch = getopt (argc, argv, "c:o:nh")) != EOF) {
 		switch (ch) {
-		case 'f':
-			config_file_name = optarg;
+		case 'c':
+			if (strlen(optarg) >= sizeof(sizeof(corosync_config_file) - 1)) {
+				fprintf(stderr, "Corosync config file path too long\n");
+				exit(1);
+			}
+			strncpy(corosync_config_file, optarg, sizeof(corosync_config_file) - 1);
 			break;
 		case 'o':
 			output_file_name = optarg;
 			break;
 		case 'n':
-			nosync = 1;
+			sync_cmds = 0;
 			break;
 		default:
 			usage(argv[0]);
@@ -675,23 +777,18 @@ int main(int argc, char **argv)
 		}
 	}
 
-	if (config_file_name) {
-		sprintf(envstring, "COROSYNC_MAIN_CONFIG_FILE=%s", config_file_name);
-		putenv(envstring);
-	}
 	if (output_file_name) {
 		output_file = fopen(output_file_name, "w");
 		if (!output_file) {
 			fprintf(stderr, "Unable to open %s for output: %s\n", output_file_name, strerror(errno));
-			exit(-1);
+			exit(3);
 		}
 	}
 	else {
 		output_file = stdout;
 	}
-#ifndef HAVE_READLINE_READLINE_H
+
 	is_tty = isatty(STDIN_FILENO);
-#endif
 
 	qb_log_filter_ctl(QB_LOG_SYSLOG, QB_LOG_FILTER_ADD,
 			  QB_LOG_FILTER_FUNCTION, "*", LOG_DEBUG);
@@ -710,9 +807,26 @@ int main(int argc, char **argv)
 			   sigchld_handler,
 			   &sigchld_qb_handle);
 
-	/* Create a full cluster of nodes from corosync.conf */
+
+#ifdef HAVE_READLINE_READLINE_H
+	/* Readline will deal with completed lines when they arrive */
+	/*
+	 * For scripting add '#' to the start of the prompt so that
+	 * parsers can ignore input lines
+	 */
+	rl_already_prompted = 1;
+	if (is_tty) {
+		rl_callback_handler_install("vqsim> ", parse_input_command);
+	} else {
+		rl_callback_handler_install("#vqsim> ", parse_input_command);
+	}
+#endif
+
+
+
+/* Create a full cluster of nodes from corosync.conf */
 	read_corosync_conf();
-	if (create_nodes_from_config() && !nosync) {
+	if (create_nodes_from_config() && sync_cmds) {
 		/* Delay kb input handling by 1 second when we've just
 		   added the nodes from corosync.conf; expect that
 		   the delay will be cancelled substantially earlier
@@ -724,8 +838,9 @@ int main(int argc, char **argv)
 				  NULL,
 				  start_kb_input_timeout,
 				  &kb_timer);
+		waiting_for_sync = 1;
 	} else {
-		start_kb_input();
+		resume_kb_input(0);
 	}
 
 	qb_loop_run(poll_loop);
